import copy

from . import basic_parser
from bs4 import BeautifulSoup

SECTION_ID = {
    'noun': 'Noun',
    'verb': 'Verb',
    'adverb': 'Adverb',
    'adjective': 'Adjective',
    'preposition': 'Preposition',
    'numeral': 'Numeral',
    'proper_noun': 'Proper_noun',
    'pronoun': 'Pronoun',
    'particle': 'Particle',
    'interjection': 'Interjection',
    'phrase': 'Phrase',
    'contraction': 'Contraction',
    'conjunction': 'Conjunction',
    'article': 'Article',
    'suffix': 'Suffix',
    'abbreviation': 'Abbreviation',
    'initialism': 'Initialism',
    'proverb': 'Proverb',
    'prefix': 'Prefix'
}


class EnglishParser(basic_parser.BasicParser):
    def parse(self):
        if not self.html:
            raise ValueError()
        word = {'meanings': self.get_meanings()}
        return word

    def get_meanings(self, meaning_types=list(SECTION_ID.keys()), get_examples=True, get_empty_meaning_types=False):
        return self._get_meanings(self._get_language_soup('English'), meaning_types, get_examples,
                                  get_empty_meaning_types)

    def _get_meanings(self, soup, meaning_types=list(SECTION_ID.keys()), get_examples=True, get_empty_meaning_types=False):
        meanings = {}
        if get_empty_meaning_types:
            for meaning_type in list(SECTION_ID.keys()):
                meanings[meaning_type] = []
        if soup:
            soup_copy = copy.copy(soup)
            for meaning_type in meaning_types:
                if meaning_type in SECTION_ID:
                    meaning_list = []
                    title = soup_copy.find(id=SECTION_ID[meaning_type])
                    if title:
                        for li_meaning in title.find_next('ol').find_all('li', recursive=False):
                            examples = []
                            example_list = li_meaning.find('dl')
                            if example_list:
                                example_list = example_list.extract()
                                if get_examples:
                                    for li_example in example_list.find_all('dd'):
                                        if li_example.text.rstrip():
                                            examples.append(li_example.text.rstrip())
                            meaning_list.append({
                                'meaning': li_meaning.text.rstrip(),
                                'examples': examples
                            })
                    if get_empty_meaning_types or meaning_list:
                        meanings[meaning_type] = meaning_list
        return meanings

    def _get_language_soup(self, language_section_id):
        language_tag = self.soup.find(id=language_section_id)
        if language_tag:
            english_section_html = ''
            for el in self.soup.find(id=language_section_id).parent.next_siblings:
                if el.name == 'h2':
                    break
                english_section_html += str(el)
            return BeautifulSoup(english_section_html, 'html.parser')
        else:
            return None



import requests

import pywiktionary
import pywiktionary.parsers.basic_parser
import pywiktionary.parsers.english_parser
import pywiktionary.parsers.italian_parser


LANGUAGE_CODES = {
    'italian': 'it',
    'it': 'it',
    'english': 'en',
    'en': 'en'
}

LANGUAGE_PARSERS = {
    'it': pywiktionary.parsers.italian_parser.ItalianParser,
    'en': pywiktionary.parsers.english_parser.EnglishParser
}


class WiktionaryParserFactory:
    def __init__(self, default_language='en', default_parser_class=None):
        # TODO: Raise an exception when a language not implemented yet has been passed AND the default parser class is not passed
        self._base_url = 'https://{lang_code}.wiktionary.org/w/index.php?printable=yes&title={page_title}'
        self.default_language = LANGUAGE_CODES[default_language] if default_language in LANGUAGE_CODES else default_language
        if default_parser_class:
            self.default_parser_class = default_parser_class
        else:
            if self.default_language in LANGUAGE_PARSERS:
                self.default_parser_class = LANGUAGE_PARSERS[self.default_language]
            else:
                self.default_parser_class = pywiktionary.parsers.basic_parser.BasicParser

    def get_page(self, title, language=None):
        if not language or language not in LANGUAGE_CODES:
            language = self.default_language
        wiktionary_response = requests.get(self._format_url(language=language, title=title))
        if wiktionary_response.status_code == 404:
            raise PageNotFoundException
        return self.default_parser_class(wiktionary_response.html)

    def _format_url(self, language, title):
        return self._base_url.format(lang_code=language, page_title=title)


class PageNotFoundException(requests.RequestException):
    pass



from . import basic_parser
from bs4 import BeautifulSoup
import copy


SECTION_ID = {
    'noun': 'Sostantivo',
    'verb': 'Verbo',
    'adverb': 'Avverbio',
    'adjective': 'Aggettivo',
    'preposition': 'Preposizione',
    'numeral_ordinal': 'Ordinale',
    'numeral_cardinal': 'Cardinale',
    'proper_noun': 'Nome_proprio',
    'pronoun': 'Pronome',
    'particle': 'Particella',
    'interjection': 'Interiezione',
    'phrase': 'Frase',
    'contraction': 'Contrazione',
    'conjunction': 'Congiunzione',
    'article': 'Articolo',
    'suffix': 'Suffisso',
    'abbreviation': 'Acronimo_/_Abbreviazione',
    'chat_speaking': 'Abbreviazione_in_uso_nelle_chat',
    'proverb': 'Proverbio',
    'prefix': 'Prefisso'
}


class ItalianParser(basic_parser.BasicParser):
    def parse(self):
        if not self.html:
            raise ValueError()
        word = {'meanings': self.get_meanings()}
        return word

    def get_meanings(self, meaning_types=list(SECTION_ID.keys()), get_examples=True, get_empty_meaning_types=False):
        return self._get_meanings(self._get_language_soup('Italiano'), meaning_types, get_examples, get_empty_meaning_types)

    def _get_meanings(self, soup, meaning_types=list(SECTION_ID.keys()), get_examples=True, get_empty_meaning_types=False):
        meanings = {}
        if get_empty_meaning_types:
            for meaning_type in list(SECTION_ID.keys()):
                meanings[meaning_type] = []
        if soup:
            soup_copy = copy.copy(soup)
            for meaning_type in meaning_types:
                if meaning_type in SECTION_ID:
                    meaning_list = []
                    title = soup_copy.find(id=SECTION_ID[meaning_type])
                    if title:
                        for li_meaning in title.find_next('ol').find_all('li', recursive=False):
                            examples = []
                            example_list = li_meaning.find('ul')
                            if example_list:
                                example_list = example_list.extract()
                                if get_examples:
                                    for li_example in example_list.find_all('li'):
                                        if li_example.text.rstrip():
                                            examples.append(li_example.text.rstrip())
                            meaning_list.append({
                                'meaning': li_meaning.text.rstrip(),
                                'examples': examples
                            })
                    if get_empty_meaning_types or meaning_list:
                        meanings[meaning_type] = meaning_list
        return meanings

    def _get_language_soup(self, language_section_id):
        language_tag = self.soup.find(id=language_section_id)
        if language_tag:
            italian_section_html = ''
            for el in self.soup.find(id=language_section_id).parent.next_siblings:
                if el.name == 'h2':
                    break
                italian_section_html += str(el)
            return BeautifulSoup(italian_section_html, 'html.parser')
        else:
            return None
